@@ -1,9 +1,8 @@
-#include "bsdtar_platform.h"
-
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/stat.h>
 
+#include <ctype.h>
 #include <fcntl.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -16,11 +15,31 @@
 #include "insecure_memzero.h"
 #include "keyfile.h"
 #include "keygen.h"
+#include "passphrase_entry.h"
 #include "readpass.h"
 #include "sysendian.h"
 #include "tarsnap_opt.h"
 #include "warnp.h"
 
+static int
+check_printable(const char * str)
+{
+	size_t i;
+
+	/*
+	 * Bail if there's any unprintable character in the "C" locale.  This
+	 * assumes that we haven't run setlocale(), so this process is using
+	 * the "C" locale by default (as specified in C99).
+	 */
+	for (i = 0; i < strlen(str); i++) {
+		if (!isprint(str[i]))
+			return (1);
+	}
+
+	/* Success! */
+	return (0);
+}
+
 int
 keygen_actual(struct register_internal * C, const char * keyfilename,
 	const int passphrased, const uint64_t maxmem,
@@ -52,6 +71,13 @@ keygen_actual(struct register_internal * C, const char * keyfilename,
 		goto err0;
 	}
 
+	/* The machine name must be printable. */
+	if (check_printable(C->name)) {
+		warn0("Machine name must be printable 7-bit ASCII: %s",
+		    C->name);
+		goto err0;
+	}
+
 	/* Sanity-check the memory size. */
 	if (maxmem > SIZE_MAX) {
 		fprintf(stderr, "Passphrase memory size is too large\n");
@@ -93,7 +119,8 @@ keygen_actual(struct register_internal * C, const char * keyfilename,
 		if (keyfile_read(oldkeyfilename, &dummy,
 		    CRYPTO_KEYMASK_HMAC_CHUNK |
 		    CRYPTO_KEYMASK_HMAC_NAME |
-		    CRYPTO_KEYMASK_HMAC_CPARAMS, 0)) {
+		    CRYPTO_KEYMASK_HMAC_CPARAMS, 0,
+		    PASSPHRASE_TTY_STDIN, NULL)) {
 			warnp("Error reading old key file");
 			goto err3;
 		}
@@ -170,5 +197,3 @@ err0:
 	/* Failure! */
 	return (-1);
 }
-
-
