@@ -1,6 +1,7 @@
 #include <stdint.h>
 #include <string.h>
 
+#include "crypto.h"
 #include "crypto_dh.h"
 #include "crypto_verify_bytes.h"
 #include "keygen.h"
@@ -14,6 +15,10 @@ static sendpacket_callback callback_register_send;
 static handlepacket_callback callback_register_challenge;
 static handlepacket_callback callback_register_response;
 
+/**
+ * keygen_network_register(C):
+ * Register a new machine key with the Tarsnap server.
+ */
 int
 keygen_network_register(struct register_internal * C)
 {
