@@ -2,6 +2,7 @@
 #include <stddef.h>
 
 #include <openssl/bn.h>
+#include <openssl/crypto.h>
 #include <openssl/err.h>
 #include <openssl/evp.h>
 #include <openssl/opensslv.h>
@@ -24,16 +25,6 @@
 /* Compatibility for LibreSSL 2.7.0+: pretend to be OpenSSL 1.1.0. */
 #define OPENSSL_VERSION_NUMBER 0x1010000fL
 
-/*
- * To free the shared memory in 2.7.0+, we need to run EVP_cleanup() in
- * crypto_compat_free().  This function is documented as being deprecated on
- * LibreSSL 2.7.0+ (and a no-op on OpenSSL 1.1.0+), but LibreSSL's
- * crypto/evp/names.c clearly shows that EVP_cleanup() does stuff, and
- * valgrind memory checks show that we need to call it manually.
- * (Checked in LibreSSL 2.7.0 and 3.4.2.)
- */
-#define NEED_EVP_CLEANUP
-
 #else
 /* Compatibility for LibreSSL before 2.7.0: pretend to be OpenSSL 1.0.1g. */
 #define OPENSSL_VERSION_NUMBER 0x1000107fL
@@ -43,21 +34,21 @@
 #endif /* LIBRESSL_VERSION_NUMBER */
 
 /**
- * crypto_compat_RSA_valid_size(rsa):
- * Return nonzero if ${rsa} has a valid size, and zero for an invalid size.
+ * crypto_compat_RSA_valid_size(key):
+ * Return nonzero if ${key} has a valid size, and zero for an invalid size.
  */
 int
-crypto_compat_RSA_valid_size(const RSA * const rsa)
+crypto_compat_RSA_valid_size(const RSA * const key)
 {
 
 	/* Sanity checks. */
-	assert(rsa != NULL);
+	assert(key != NULL);
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-	assert(rsa->n != NULL);
-	return ((RSA_size(rsa) == 256) && (BN_num_bits(rsa->n) == 2048));
+	assert(key->n != NULL);
+	return ((RSA_size(key) == 256) && (BN_num_bits(key->n) == 2048));
 #else
-	return ((RSA_size(rsa) == 256) && (RSA_bits(rsa) == 2048));
+	return ((RSA_size(key) == 256) && (RSA_bits(key) == 2048));
 #endif
 }
 
@@ -197,7 +188,7 @@ crypto_compat_RSA_export(RSA * key, const BIGNUM ** n, const BIGNUM ** e,
  * Generate a key pair.
  */
 RSA *
-crypto_compat_RSA_generate_key()
+crypto_compat_RSA_generate_key(void)
 {
 	RSA * key;
 
@@ -254,23 +245,34 @@ void
 crypto_compat_free(void)
 {
 
+#if !defined(LIBRESSL_VERSION_NUMBER) && OPENSSL_VERSION_NUMBER > 0x1010000fL
+	/* OpenSSL 1.1.0 and higher: do nothing; the library uses atexit(). */
+#elif defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER > 0x3070100fL
+	/* LibreSSL 3.7.1 and higher. */
+	OPENSSL_cleanup();
+#elif defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER > 0x3060000fL
+	/* LibreSSL 3.6.0 to 3.7.0. */
+	OPENSSL_cleanup();
+	CRYPTO_cleanup_all_ex_data();
+#else
+	/* Earlier versions of OpenSSL and LibreSSL. */
+
 	/* Free OpenSSL error queue. */
 #if OPENSSL_VERSION_NUMBER < 0x10000000L
 	ERR_remove_state(0);
 #elif OPENSSL_VERSION_NUMBER < 0x10100000L
 	ERR_remove_thread_state(NULL);
-#else
-	/* Nothing needed for OpenSSL >= 1.1. */
 #endif
 
 	/* Free OpenSSL error strings. */
 	ERR_free_strings();
 
-#ifdef NEED_EVP_CLEANUP
-	/* Additional cleaning needed for some versions of LibreSSL. */
+#if LIBRESSL_VERSION_NUMBER >= 0x2070000fL
+	/* Additional cleaning needed for LibreSSL 2.7.0 to 3.5.x. */
 	EVP_cleanup();
 #endif
 
 	/* A more general OpenSSL cleanup function. */
 	CRYPTO_cleanup_all_ex_data();
+#endif
 }
