@@ -1,13 +1,13 @@
-#ifndef _CRYPTO_COMPAT_H_
-#define _CRYPTO_COMPAT_H_
+#ifndef CRYPTO_COMPAT_H_
+#define CRYPTO_COMPAT_H_
 
 #include <openssl/ossl_typ.h>
 
 /**
- * crypto_compat_RSA_valid_size(rsa):
- * Return nonzero if ${rsa} has a valid size, and zero for an invalid size.
+ * crypto_compat_RSA_valid_size(key):
+ * Return nonzero if ${key} has a valid size, and zero for an invalid size.
  */
-int crypto_compat_RSA_valid_size(const RSA * const key);
+int crypto_compat_RSA_valid_size(const RSA * const);
 
 /**
  * crypto_compat_RSA_import(key, n, e, d, p, q, dmp1, dmq1, iqmp):
@@ -15,8 +15,8 @@ int crypto_compat_RSA_valid_size(const RSA * const key);
  * free any any BIGNUMs which have not been imported into the ${key}, but do
  * not free the ${key} itself.
  */
-int crypto_compat_RSA_import(RSA * key, BIGNUM * n, BIGNUM * e, BIGNUM * d,
-    BIGNUM * p, BIGNUM * q, BIGNUM * dmp1, BIGNUM * dmq1, BIGNUM * iqmp);
+int crypto_compat_RSA_import(RSA *, BIGNUM *, BIGNUM *, BIGNUM *,
+    BIGNUM *, BIGNUM *, BIGNUM *, BIGNUM *, BIGNUM *);
 
 /**
  * crypto_compat_RSA_export(key, n, e, d, p, q, dmp1, dmq1, iqmp):
@@ -24,9 +24,9 @@ int crypto_compat_RSA_import(RSA * key, BIGNUM * n, BIGNUM * e, BIGNUM * d,
  * must be non-NULL; the other values may be NULL if desired, and will
  * therefore not be exported.
  */
-int crypto_compat_RSA_export(RSA * key, const BIGNUM ** n, const BIGNUM ** e,
-    const BIGNUM ** d, const BIGNUM ** p, const BIGNUM ** q,
-    const BIGNUM ** dmp1, const BIGNUM ** dmq1, const BIGNUM ** iqmp);
+int crypto_compat_RSA_export(RSA *, const BIGNUM **, const BIGNUM **,
+    const BIGNUM **, const BIGNUM **, const BIGNUM **,
+    const BIGNUM **, const BIGNUM **, const BIGNUM **);
 
 /**
  * crypto_compat_RSA_generate_key(void):
@@ -40,4 +40,4 @@ RSA * crypto_compat_RSA_generate_key(void);
  */
 void crypto_compat_free(void);
 
-#endif /* !_CRYPTO_COMPAT_H_ */
+#endif /* !CRYPTO_COMPAT_H_ */
