@@ -1,5 +1,5 @@
-#ifndef _CRYPTO_INTERNAL_H_
-#define _CRYPTO_INTERNAL_H_
+#ifndef CRYPTO_INTERNAL_H_
+#define CRYPTO_INTERNAL_H_
 
 #include <stdint.h>
 
@@ -91,10 +91,17 @@ void crypto_keys_subr_free_HMAC(struct crypto_hmac_key **);
  */
 int crypto_file_init_keys(void);
 
+/**
+ * crypto_keys_init_keycache(void):
+ * Initialize the key cache.
+ */
+int crypto_keys_init_keycache(void);
+
 /**
  * crypto_MGF1(seed, seedlen, buf, buflen):
- * The MGF1 mask generation function, as specified in RFC 3447.
+ * The MGF1 mask generation function, as specified in RFC 3447 section B.2.1,
+ * using SHA256 as the hash function.
  */
 void crypto_MGF1(uint8_t *, size_t, uint8_t *, size_t);
 
-#endif /* !_CRYPTO_INTERNAL_H_ */
+#endif /* !CRYPTO_INTERNAL_H_ */
