@@ -1,5 +1,5 @@
-#ifndef _NETPROTO_H_
-#define _NETPROTO_H_
+#ifndef NETPROTO_H_
+#define NETPROTO_H_
 
 #include <stddef.h>
 #include <stdint.h>
@@ -15,14 +15,14 @@ typedef struct netproto_connection_internal NETPROTO_CONNECTION;
     (NETWORK_STATUS_MAX + 1)		/* Protocol error. */
 
 /**
- * _netproto_printerr(status):
+ * netproto_printerr_internal(status):
  * Print the error message associated with the given status code.
  */
-void _netproto_printerr(int);
+void netproto_printerr_internal(int);
 
 #define netproto_printerr(x)	do {	\
 	warnline;			\
-	_netproto_printerr(x);		\
+	netproto_printerr_internal(x);	\
 } while (0)
 
 /**
@@ -79,4 +79,4 @@ int netproto_flush(NETPROTO_CONNECTION *);
  */
 int netproto_close(NETPROTO_CONNECTION *);
 
-#endif /* !_NETPROTO_H_ */
+#endif /* !NETPROTO_H_ */
