@@ -1,5 +1,3 @@
-#include "bsdtar_platform.h"
-
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -304,6 +302,8 @@ dh_received(void * cookie, int status)
 		goto err2;
 	case 1:
 		/* Bad signature. */
+		warn0("Diffie-Hellman parameter from server is not validly"
+		    " signed");
 		status = NETPROTO_STATUS_PROTERR;
 		goto err1;
 	case 0:
