@@ -140,7 +140,8 @@ err0:
 }
 
 /**
- * _network_writeq_add(Q, buf, buflen, timeo, callback, cookie, abstimeo):
+ * network_writeq_add_internal(Q, buf, buflen, timeo, callback, cookie,
+ *     abstimeo):
  * Add a buffer write to the specified write queue.  The callback function
  * will be called when the write is finished, fails, or is cancelled.
  * If ${abstimeo} is zero, the timeout is relative to when the buffer in
@@ -150,9 +151,9 @@ err0:
  * point, with a status of NETWORK_STATUS_ZEROBYTE.
  */
 int
-_network_writeq_add(NETWORK_WRITEQ * Q, const uint8_t * buf, size_t buflen,
-    struct timeval * timeo, network_callback * callback, void * cookie,
-    int abstimeo)
+network_writeq_add_internal(NETWORK_WRITEQ * Q, const uint8_t * buf,
+    size_t buflen, struct timeval * timeo, network_callback * callback,
+    void * cookie, int abstimeo)
 {
 	struct network_writeq_buf * QB;
 	struct network_writeq_buf ** tailptr_old;
