@@ -1,5 +1,5 @@
 /*-
- * Copyright 2009 Colin Percival
+ * Copyright 2009-2025 Tarsnap Backup Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,8 +26,8 @@
  * This file was originally written by Colin Percival as part of the Tarsnap
  * online backup system.
  */
-#ifndef _SCRYPTENC_H_
-#define _SCRYPTENC_H_
+#ifndef SCRYPTENC_H_
+#define SCRYPTENC_H_
 
 #include <stdint.h>
 #include <stdio.h>
@@ -151,7 +151,8 @@ int scryptdec_file(FILE *, FILE *, const uint8_t *, size_t,
     struct scryptenc_params *, int, int);
 
 /**
- * scryptdec_file_prep(infile, passwd, passwdlen, params, force, cookie):
+ * scryptdec_file_prep(infile, passwd, passwdlen, params, verbose, force,
+ *     cookie):
  * Prepare to decrypt ${infile}, including checking the passphrase.  Allocate
  * a cookie at ${cookie}.  After calling this function, ${infile} should not
  * be modified until the decryption is completed by scryptdec_file_copy().
@@ -177,4 +178,4 @@ int scryptdec_file_copy(struct scryptdec_file_cookie *, FILE *);
  */
 void scryptdec_file_cookie_free(struct scryptdec_file_cookie *);
 
-#endif /* !_SCRYPTENC_H_ */
+#endif /* !SCRYPTENC_H_ */
