@@ -1,9 +1,6 @@
-#include "bsdtar_platform.h"
-
 #include <sys/time.h>
 
 #include "monoclock.h"
-#include "warnp.h"
 
 #include "tvmath.h"
 
