@@ -1,5 +1,5 @@
 /*-
- * Copyright 2006-2009 Colin Percival
+ * Copyright 2006-2025 Tarsnap Backup Inc.
  * All rights reserved.
  *
  * Portions of the file below are covered by the following license:
@@ -78,6 +78,7 @@ __FBSDID("$FreeBSD: src/usr.bin/tar/bsdtar.c,v 1.93 2008/11/08 04:43:24 kientzle
 #include <assert.h>
 
 #include "bsdtar.h"
+#include "chunks.h"
 #include "crypto.h"
 #include "dirutil.h"
 #include "humansize.h"
@@ -152,10 +153,17 @@ bsdtar_init(void)
 	bsdtar->modestr = NULL;
 	bsdtar->option_csv_filename = NULL;
 	bsdtar->configfiles = NULL;
+	bsdtar->keyfile = NULL;
+	bsdtar->option_passphrase_arg = NULL;
 	bsdtar->archive = NULL;
 	bsdtar->progname = NULL;
 	bsdtar->argv = NULL;
 	bsdtar->optarg = NULL;
+	bsdtar->conffile = NULL;
+	bsdtar->conf_opt = NULL;
+	bsdtar->conf_arg = NULL;
+	bsdtar->conffile_actual = NULL;
+	bsdtar->conffile_buffer = NULL;
 	bsdtar->write_cookie = NULL;
 	bsdtar->chunk_cache = NULL;
 	bsdtar->argv_orig = NULL;
@@ -170,13 +178,6 @@ bsdtar_init(void)
 	bsdtar->uname_cache = NULL;
 	bsdtar->siginfo = NULL;
 	bsdtar->substitution = NULL;
-	bsdtar->keyfile = NULL;
-	bsdtar->conffile = NULL;
-	bsdtar->conf_opt = NULL;
-	bsdtar->conf_arg = NULL;
-	bsdtar->conffile_actual = NULL;
-	bsdtar->conffile_buffer = NULL;
-	bsdtar->option_passphrase_arg = NULL;
 
 	/* Initialize temporary tapenames array. */
 	bsdtar->tapenames_setup = strlist_init(0);
@@ -228,8 +229,9 @@ bsdtar_atexit(void)
 	/* Free file-parsing variables from util.c. */
 	free(bsdtar->conffile_buffer);
 	if ((bsdtar->conffile_actual != NULL) &&
-	    (bsdtar->conffile_actual != stdin))
-		fclose(bsdtar->conffile_actual);
+	    (bsdtar->conffile_actual != stdin) &&
+	    fclose(bsdtar->conffile_actual))
+		bsdtar_warnc(bsdtar, errno, "fclose");
 
 	/* Free matching and (if applicable) substitution patterns. */
 	cleanup_exclusions(bsdtar);
@@ -255,6 +257,7 @@ main(int argc, char **argv)
 	size_t			 i;
 	int			 j;
 	char			*tapename_cmdline;
+	char			*xdg_configdir;
 
 	WARNP_INIT;
 
@@ -392,7 +395,7 @@ main(int argc, char **argv)
 			break;
 		case OPTION_ARCHIVE_NAMES:
 			if (bsdtar->option_archive_names_set)
-				bsdtar_errc(bsdtar, 1, errno,
+				bsdtar_errc(bsdtar, 1, 0,
 				    "Two --archive-names options given");
 			if (bsdtar->optarg == NULL)
 				bsdtar_errc(bsdtar, 1, 0,
@@ -442,7 +445,7 @@ main(int argc, char **argv)
 			break;
 		case OPTION_CSV_FILE: /* tarsnap */
 			if (bsdtar->option_csv_filename != NULL)
-				bsdtar_errc(bsdtar, 1, errno,
+				bsdtar_errc(bsdtar, 1, 0,
 				    "Two --csv-file options given");
 			if ((bsdtar->option_csv_filename = strdup(
 			    bsdtar->optarg)) == NULL)
@@ -461,8 +464,17 @@ main(int argc, char **argv)
 			optq_push(bsdtar, "disk-pause", bsdtar->optarg);
 			break;
 		case OPTION_DRYRUN: /* tarsnap */
+			if (bsdtar->option_dryrun != 0)
+				bsdtar_errc(bsdtar, 1, 0,
+				    "Can only specify one --dry-run* option");
 			bsdtar->option_dryrun = 1;
 			break;
+		case OPTION_DRYRUN_METADATA: /* tarsnap */
+			if (bsdtar->option_dryrun != 0)
+				bsdtar_errc(bsdtar, 1, 0,
+				    "Can only specify one --dry-run* option");
+			bsdtar->option_dryrun = 2;
+			break;
 		case OPTION_EXCLUDE: /* GNU tar */
 			optq_push(bsdtar, "exclude", bsdtar->optarg);
 			break;
@@ -490,6 +502,9 @@ main(int argc, char **argv)
 			/* Hack: -h by itself is the "help" command. */
 			possible_help_request = 1;
 			break;
+		case OPTION_HASHES: /* tarsnap */
+			bsdtar->option_hashes = 1;
+			break;
 		case OPTION_HELP: /* GNU tar, others */
 			long_help(bsdtar);
 			exit(0);
@@ -606,6 +621,9 @@ main(int argc, char **argv)
 				    ARCHIVE_STAT_MTIME_NANOS(&st);
 			}
 			break;
+		case OPTION_NOATIME: /* tarsnap */
+			optq_push(bsdtar, "noatime", NULL);
+			break;
 		case OPTION_NODUMP: /* star */
 			optq_push(bsdtar, "nodump", NULL);
 			break;
@@ -651,6 +669,9 @@ main(int argc, char **argv)
 		case OPTION_NO_MAXBW_RATE_UP:
 			optq_push(bsdtar, "no-maxbw-rate-up", NULL);
 			break;
+		case OPTION_NO_NOATIME:
+			optq_push(bsdtar, "no-noatime", NULL);
+			break;
 		case OPTION_NO_NODUMP:
 			optq_push(bsdtar, "no-nodump", NULL);
 			break;
@@ -688,7 +709,11 @@ main(int argc, char **argv)
 			set_mode(bsdtar, opt, "--nuke");
 			break;
 		case OPTION_NULL: /* GNU tar */
-			bsdtar->option_null++;
+		case OPTION_NULL_INPUT: /* tarsnap */
+			bsdtar->option_null_input++;
+			break;
+		case OPTION_NULL_OUTPUT: /* tarsnap */
+			bsdtar->option_null_output++;
 			break;
 		case OPTION_NUMERIC_OWNER: /* GNU tar */
 			bsdtar->option_numeric_owner++;
@@ -896,6 +921,35 @@ main(int argc, char **argv)
 			bsdtar->conffile = NULL;
 		}
 
+		/* Check if ${XDG_CONFIG_HOME} is set. */
+		if ((xdg_configdir = getenv("XDG_CONFIG_HOME")) != NULL) {
+			/*
+			 * If it exists, use
+			 * ${XDG_CONFIG_HOME}/tarsnap/tarsnap.conf
+			 */
+			if (asprintf(&bsdtar->conffile,
+			    "%s/tarsnap/tarsnap.conf", xdg_configdir) == -1)
+				bsdtar_errc(bsdtar, 1, errno, "No memory");
+		} else if (bsdtar->homedir != NULL) {
+			/*
+			 * If it doesn't exist, use
+			 * $HOME/.config/tarsnap/tarsnap.conf
+			 */
+			if (asprintf(&bsdtar->conffile,
+			    "%s/.config/tarsnap/tarsnap.conf", bsdtar->homedir)
+			    == -1)
+				bsdtar_errc(bsdtar, 1, errno, "No memory");
+		}
+
+		/* Read tarsnap.conf (if applicable). */
+		if (bsdtar->conffile != NULL) {
+			configfile(bsdtar, bsdtar->conffile, 0);
+
+			/* Free string allocated by asprintf. */
+			free(bsdtar->conffile);
+			bsdtar->conffile = NULL;
+		}
+
 		/* Process options from system-wide tarsnap.conf. */
 		configfile(bsdtar, ETC_TARSNAP_CONF, 0);
 	}
@@ -920,6 +974,7 @@ main(int argc, char **argv)
 		    "Archive name must be specified");
 	if ((bsdtar->ntapes > 1) &&
 	    (bsdtar->mode != OPTION_PRINT_STATS &&
+	     bsdtar->mode != OPTION_LIST_ARCHIVES &&
 	     bsdtar->mode != 'd'))
 		bsdtar_errc(bsdtar, 1, 0,
 		    "Option -f may only be specified once in mode %s",
@@ -943,6 +998,25 @@ main(int argc, char **argv)
 		bsdtar_errc(bsdtar, 1, 0,
 		    "Cache directory must be specified for %s",
 		    bsdtar->modestr);
+	if (bsdtar->mode == 'd' ||
+	    bsdtar->mode == OPTION_RECOVER ||
+	    bsdtar->mode == OPTION_PRINT_STATS) {
+		switch (chunks_directory_exists(bsdtar->cachedir)) {
+		case 1:
+			/* Exists; good. */
+			break;
+		case 0:
+			/* Does not exist; error and exit. */
+			bsdtar_errc(bsdtar, 1, 0,
+			    "Cache directory must already exist for %s;"
+			    " check --cachedir and/or run --fsck",
+			    bsdtar->modestr);
+		case -1:
+			/* Could not check; error and exit. */
+			bsdtar_errc(bsdtar, 1, errno,
+			    "Could not check if cachedir exists");
+		}
+	}
 	if (tarsnap_opt_aggressive_networking != 0) {
 		if ((bsdtar->bwlimit_rate_up != 0) ||
 		    (bsdtar->bwlimit_rate_down != 0)) {
@@ -953,15 +1027,28 @@ main(int argc, char **argv)
 			tarsnap_opt_aggressive_networking = 0;
 		}
 	}
+	if ((bsdtar->option_dryrun == 2) && bsdtar->option_print_stats)
+		bsdtar_errc(bsdtar, 1, 0, "--dry-run-metadata is "
+		    "incompatible with --print-stats");
 
 	/*
-	 * The -f option doesn't make sense for --list-archives, --fsck,
-	 * --fsck-prune, or --nuke.
+	 * The -f option doesn't make sense for --fsck, --fsck-prune, or
+	 * --nuke.  It can be used with --list-archives --hash, but not
+	 * --list-archives on its own; sanity-checking that detail is
+	 * done in tarsnap_mode_list_archives().
 	 */
 	if ((bsdtar->ntapes > 0) &&
+	    (bsdtar->mode != OPTION_LIST_ARCHIVES) &&
 	    (bsdtar->mode != OPTION_PRINT_STATS))
 		only_mode(bsdtar, "-f", "cxtdr");
 
+#ifndef O_NOATIME
+	if (bsdtar->option_noatime)
+		bsdtar_warnc(bsdtar, 0,
+		    "noatime requested, but not supported by this OS; "
+		    "ignoring option");
+#endif
+
 	/*
 	 * These options don't make sense for the "delete" and "convert to
 	 * tar" modes.
@@ -976,8 +1063,14 @@ main(int argc, char **argv)
 		only_mode(bsdtar, "--newer-mtime", "cxt");
 	if (bsdtar->option_absolute_paths)
 		only_mode(bsdtar, "-P", "cxt");
-	if (bsdtar->option_null)
-		only_mode(bsdtar, "--null", "cxt");
+	if (bsdtar->option_null_input)
+		only_mode(bsdtar, "--null-input", "cxt");
+	if (bsdtar->option_null_output) {
+		/* Allow in --list-archives, --print-stats, and xt modes. */
+		if ((bsdtar->mode != OPTION_LIST_ARCHIVES) &&
+		    (bsdtar->mode != OPTION_PRINT_STATS))
+			only_mode(bsdtar, "--null-output", "xt");
+	}
 
 	/* We should only have remaining args in -c, -t, and -x modes. */
 	if (bsdtar->argc != 0) {
@@ -1003,8 +1096,10 @@ main(int argc, char **argv)
 		only_mode(bsdtar, "-U", "x");
 	if (bsdtar->option_warn_links)
 		only_mode(bsdtar, "--check-links", "c");
-	if (bsdtar->option_dryrun)
+	if (bsdtar->option_dryrun == 1)
 		only_mode(bsdtar, "--dry-run", "c");
+	if (bsdtar->option_dryrun == 2)
+		only_mode(bsdtar, "--dry-run-metadata", "c");
 
 	/* Check other parameters only permitted in certain modes. */
 	if (bsdtar->symlink_mode != '\0') {
@@ -1195,7 +1290,7 @@ main(int argc, char **argv)
 		tarsnap_mode_recover(bsdtar, 0);
 		break;
 	case OPTION_LIST_ARCHIVES:
-		tarsnap_mode_list_archives(bsdtar);
+		tarsnap_mode_list_archives(bsdtar, bsdtar->option_hashes);
 		break;
 	case OPTION_NUKE:
 		tarsnap_mode_nuke(bsdtar);
@@ -1230,9 +1325,17 @@ main(int argc, char **argv)
 		chdir(cachedir);
 #endif
 
-	if (bsdtar->return_value != 0)
+	if (bsdtar->return_value != 0) {
 		bsdtar_warnc(bsdtar, 0,
 		    "Error exit delayed from previous errors.");
+		/* If we modified the storage, return 2 instead, and warn. */
+		if (bsdtar->storage_modified) {
+			bsdtar->return_value = 2;
+			bsdtar_warnc(bsdtar, 0,
+			    "Data on server was modified, but it might not"
+			    "be exactly what you requested");
+		}
+	}
 	return (bsdtar->return_value);
 }
 
@@ -1286,7 +1389,8 @@ usage(struct bsdtar *bsdtar)
 static void
 version(void)
 {
-	printf("tarsnap %s\n", PACKAGE_VERSION);
+
+	fprintf(stdout, "tarsnap %s\n", PACKAGE_VERSION);
 	exit(0);
 }
 
@@ -1326,7 +1430,8 @@ long_help(struct bsdtar *bsdtar)
 	fflush(stderr);
 
 	p = (strcmp(prog, "tarsnap") != 0) ? "(tarsnap)" : "";
-	printf("%s%s: create and manipulate archives on the Tarsnap backup service\n", prog, p);
+	fprintf(stdout, "%s%s: create and manipulate archives on the "
+	    "Tarsnap backup service\n", prog, p);
 
 	for (p = long_help_msg; *p != '\0'; p++) {
 		if (*p == '%') {
@@ -1707,6 +1812,14 @@ dooption(struct bsdtar *bsdtar, const char * conf_opt,
 			bsdtar_errc(bsdtar, 1, 0,
 			    "Invalid bandwidth rate limit: %s", conf_arg);
 		bsdtar->option_maxbw_rate_up_set = 1;
+	} else if (strcmp(conf_opt, "noatime") == 0) {
+		if (bsdtar->mode != 'c')
+			goto badmode;
+		if (bsdtar->option_noatime_set)
+			goto optset;
+
+		bsdtar->option_noatime = 1;
+		bsdtar->option_noatime_set = 1;
 	} else if (strcmp(conf_opt, "nodump") == 0) {
 		if (bsdtar->mode != 'c')
 			goto badmode;
@@ -1777,6 +1890,11 @@ dooption(struct bsdtar *bsdtar, const char * conf_opt,
 			goto optset;
 
 		bsdtar->option_maxbw_rate_up_set = 1;
+	} else if (strcmp(conf_opt, "no-noatime") == 0) {
+		if (bsdtar->option_noatime_set)
+			goto optset;
+
+		bsdtar->option_noatime_set = 1;
 	} else if (strcmp(conf_opt, "no-nodump") == 0) {
 		if (bsdtar->option_nodump_set)
 			goto optset;
@@ -1817,14 +1935,6 @@ dooption(struct bsdtar *bsdtar, const char * conf_opt,
 			goto optset;
 
 		bsdtar->option_totals_set = 1;
-	} else if (strcmp(conf_opt, "print-stats") == 0) {
-		if ((bsdtar->mode != 'c') && (bsdtar->mode != 'd'))
-			goto badmode;
-		if (bsdtar->option_print_stats_set)
-			goto optset;
-
-		bsdtar->option_print_stats = 1;
-		bsdtar->option_print_stats_set = 1;
 	} else if (strcmp(conf_opt, "passphrase") == 0) {
 		if (bsdtar->option_passphrase_entry != PASSPHRASE_UNSET)
 			goto optset;
@@ -1836,8 +1946,16 @@ dooption(struct bsdtar *bsdtar, const char * conf_opt,
 		if ((bsdtar->option_passphrase_arg = strdup(str)) == NULL)
 			bsdtar_errc(bsdtar, 1, ENOMEM,
 			    "Cannot allocate memory");
+	} else if (strcmp(conf_opt, "print-stats") == 0) {
+		if ((bsdtar->mode != 'c') && (bsdtar->mode != 'd'))
+			goto badmode;
+		if (bsdtar->option_print_stats_set)
+			goto optset;
+
+		bsdtar->option_print_stats = 1;
+		bsdtar->option_print_stats_set = 1;
 	} else if (strcmp(conf_opt, "progress-bytes") == 0) {
-		if (!((bsdtar->mode != 'c') || (bsdtar->mode != 'x')))
+		if (!((bsdtar->mode == 'c') || (bsdtar->mode == 'x')))
 			goto badmode;
 		if (bsdtar->option_progress_bytes_set)
 			goto optset;
