@@ -45,11 +45,13 @@
 #error Oops: No config.h and no built-in configuration in bsdtar_platform.h.
 #endif /* !HAVE_CONFIG_H */
 
-/* No non-FreeBSD platform will have __FBSDID, so just define it here. */
-#ifdef __FreeBSD__
-#include <sys/cdefs.h>  /* For __FBSDID */
-#else
-/* Just leaving this macro replacement empty leads to a dangling semicolon. */
+/* Get a real definition for __FBSDID if we can */
+#if HAVE_SYS_CDEFS_H
+#include <sys/cdefs.h>
+#endif
+
+/* If not, define it so as to avoid dangling semicolons. */
+#ifndef __FBSDID
 #define	__FBSDID(a)     struct _undefined_hack
 #endif
 
@@ -154,12 +156,6 @@
 #define	ARCHIVE_STAT_MTIME_NANOS(st)	(0)
 #endif
 
-/* If asprintf isn't provided by the OS, use our local version. */
-#ifndef HAVE_ASPRINTF
-#define asprintf tarsnap_asprintf
-#include "asprintf.h"
-#endif
-
 /* How to mark functions that don't return. */
 /* This facilitates use of some newer static code analysis tools. */
 #undef __LA_DEAD
