@@ -1,5 +1,5 @@
-#ifndef _CHUNKS_H_
-#define _CHUNKS_H_
+#ifndef CHUNKS_H_
+#define CHUNKS_H_
 
 #include <stddef.h>
 #include <stdio.h>
@@ -204,18 +204,18 @@ CHUNKS_S * chunks_stats_init(const char *);
 size_t chunks_stats_getdirsz(CHUNKS_S *);
 
 /**
- * chunks_stats_printglobal(stream, C, csv):
+ * chunks_stats_printglobal(stream, C, csv, print_nulls):
  * Print global statistics relating to a set of archives, optionally in ${csv}
- * format.
+ * format.  If ${print_nulls} is non-zero, use '\0' as separators.
  */
-int chunks_stats_printglobal(FILE *, CHUNKS_S *, int);
+int chunks_stats_printglobal(FILE *, CHUNKS_S *, int, int);
 
 /**
- * chunks_stats_printarchive(stream, C, name, csv):
+ * chunks_stats_printarchive(stream, C, name, csv, print_nulls):
  * Print accumulated statistics for an archive with the given name, optionally
- * in ${csv} format.
+ * in ${csv} format.  If ${print_nulls} is non-zero, use '\0' as separators.
  */
-int chunks_stats_printarchive(FILE *, CHUNKS_S *, const char *, int);
+int chunks_stats_printarchive(FILE *, CHUNKS_S *, const char *, int, int);
 
 /**
  * chunks_stats_free(C):
@@ -262,7 +262,7 @@ int chunks_stats_addchunk(CHUNKS_S *, const uint8_t *, size_t, size_t);
 void chunks_stats_extrastats(CHUNKS_S *, size_t);
 
 /*
- * Function used by --initialize-cachedir.
+ * Functions used by --initialize-cachedir and checking for a chunks directory.
  */
 
 /**
@@ -272,4 +272,11 @@ void chunks_stats_extrastats(CHUNKS_S *, size_t);
  */
 int chunks_initialize(const char *);
 
-#endif /* !_CHUNKS_H_ */
+/**
+ * chunks_directory_exists(cachepath):
+ * Return 1 if the /directory file exists within ${cachepath}, 0 if it does
+ * not, or -1 if there is an error.
+ */
+int chunks_directory_exists(const char *);
+
+#endif /* !CHUNKS_H_ */
