@@ -1,5 +1,5 @@
-#ifndef _CHUNKS_INTERNAL_H_
-#define _CHUNKS_INTERNAL_H_
+#ifndef CHUNKS_INTERNAL_H_
+#define CHUNKS_INTERNAL_H_
 
 #include <stdint.h>
 #include <stdio.h>
@@ -61,13 +61,6 @@ RWHASHTAB * chunks_directory_read(const char *, void **,
 int chunks_directory_write(const char *, RWHASHTAB *, struct chunkstats *,
     const char *);
 
-/**
- * chunks_directory_exists(cachepath):
- * Return 1 if the /directory file exists within ${cachepath}, 0 if it does
- * not, or -1 if there is an error.
- */
-int chunks_directory_exists(const char *);
-
 /**
  * chunks_directory_free(HT, dir):
  * Free the hash table ${HT} of struct chunkdata records, all of its
@@ -93,8 +86,7 @@ void chunks_stats_zero(struct chunkstats *);
  * Adjust ${stats} for the addition of ${copies} chunks each having length
  * ${len} and compressed length ${zlen}.
  */
-void chunks_stats_add(struct chunkstats *, size_t len, size_t zlen,
-    ssize_t copies);
+void chunks_stats_add(struct chunkstats *, size_t, size_t, ssize_t);
 
 /**
  * chunks_stats_addstats(to, from):
@@ -104,18 +96,19 @@ void chunks_stats_add(struct chunkstats *, size_t len, size_t zlen,
 void chunks_stats_addstats(struct chunkstats *, struct chunkstats *);
 
 /**
- * chunks_stats_printheader(stream, csv):
+ * chunks_stats_printheader(stream, csv, print_nulls):
  * Print a header line for statistics to ${stream}, optionally in ${csv}
- * format.
+ * format.  If ${print_nulls} is non-zero, use '\0' for separators.
  */
-int chunks_stats_printheader(FILE *, int);
+int chunks_stats_printheader(FILE *, int, int);
 
 /**
- * chunks_stats_print(stream, stats, name, stats_extra, csv):
+ * chunks_stats_print(stream, stats, name, stats_extra, csv, print_nulls):
  * Print a line with ${name} and combined statistics from ${stats} and
- * ${stats_extra} to ${stream}, optionally in ${csv} format.
+ * ${stats_extra} to ${stream}, optionally in ${csv} format.  If ${print_nulls}
+ * is non-zero, use '\0' as separators.
  */
 int chunks_stats_print(FILE *, struct chunkstats *, const char *,
-    struct chunkstats *, int);
+    struct chunkstats *, int, int);
 
-#endif /* !_CHUNKS_INTERNAL_H_ */
+#endif /* !CHUNKS_INTERNAL_H_ */
