@@ -121,19 +121,20 @@ archive_read_open_multitape(struct archive * a, uint64_t machinenum,
 
 /**
  * archive_write_open_multitape(a, machinenum, cachedir, tapename, argc,
- *     argv, printstats, dryrun, creationtime, csv_filename):
+ *     argv, printstats, dryrun, creationtime, csv_filename, storage_modified):
  * Open the multitape tape ${tapename} for writing and associate it with the
  * archive ${a}.  If ${printstats} is non-zero, print archive statistics when
  * the tape is closed.  If ${dryrun} is non-zero, perform a dry run.
  * Record ${creationtime} as the creation time in the archive metadata.
- * If ${csv_filename} is given, write statistics in CSV format.
+ * If ${csv_filename} is given, write statistics in CSV format.  If the
+ * data on the server has been modified, set ${*storage_modified} to 1.
  * Return a cookie which can be passed to the multitape layer.
  */
 void *
 archive_write_open_multitape(struct archive * a, uint64_t machinenum,
     const char * cachedir, const char * tapename, int argc,
     char ** argv, int printstats, int dryrun, time_t creationtime,
-    const char * csv_filename)
+    const char * csv_filename, int * storage_modified)
 {
 	struct multitape_write_internal * d;
 
@@ -142,7 +143,7 @@ archive_write_open_multitape(struct archive * a, uint64_t machinenum,
 
 	if ((d = writetape_open(machinenum, cachedir, tapename,
 	    argc, argv, printstats, dryrun, creationtime,
-	    csv_filename)) == NULL) {
+	    csv_filename, storage_modified)) == NULL) {
 		archive_set_error(a, errno, "Error creating new archive");
 		return (NULL);
 	}
@@ -318,5 +319,6 @@ nochunk:
 			return (-1);
 	}
 
+	/* Success! */
 	return (0);
 }
