@@ -7,6 +7,7 @@
 
 #include "archive.h"
 #include "ccache.h"
+#include "hexify.h"
 #include "multitape.h"
 
 #include "bsdtar.h"
@@ -31,7 +32,8 @@ tarsnap_mode_d(struct bsdtar *bsdtar)
 			    bsdtar->tapenames[i]);
 		switch (deletetape(d, bsdtar->machinenum, bsdtar->cachedir,
 		    bsdtar->tapenames[i], bsdtar->option_print_stats,
-		    bsdtar->ntapes > 1 ? 1 : 0, bsdtar->option_csv_filename)) {
+		    bsdtar->ntapes > 1 ? 1 : 0, bsdtar->option_csv_filename,
+		    &bsdtar->storage_modified)) {
 		case 0:
 			break;
 		case 1:
@@ -123,7 +125,8 @@ tarsnap_mode_print_stats(struct bsdtar *bsdtar)
 		goto err1;
 
 	/* Print statistics about the archive set. */
-	if (statstape_printglobal(d, bsdtar->option_csv_filename))
+	if (statstape_printglobal(d, bsdtar->option_csv_filename,
+	    bsdtar->option_null_output))
 		goto err2;
 
 	if (bsdtar->ntapes == 0) {
@@ -131,13 +134,15 @@ tarsnap_mode_print_stats(struct bsdtar *bsdtar)
 	} else if ((bsdtar->tapenames[0][0] == '*') &&
 	    (bsdtar->tapenames[0][1] == '\0')) {
 		/* User wants statistics on all archives. */
-		if (statstape_printall(d, bsdtar->option_csv_filename))
+		if (statstape_printall(d, bsdtar->option_csv_filename,
+		    bsdtar->option_null_output))
 			goto err2;
 	} else {
 		/* User wants statistics about specific archive(s). */
 		for (i = 0; i < bsdtar->ntapes; i++) {
 			switch (statstape_print(d, bsdtar->tapenames[i],
-			    bsdtar->option_csv_filename)) {
+			    bsdtar->option_csv_filename,
+			    bsdtar->option_null_output)) {
 			case 0:
 				break;
 			case 1:
@@ -170,17 +175,45 @@ err1:
  * Print the names of all the archives.
  */
 void
-tarsnap_mode_list_archives(struct bsdtar *bsdtar)
+tarsnap_mode_list_archives(struct bsdtar *bsdtar, int print_hashes)
 {
 	TAPE_S * d;
+	uint8_t hash[32];
+	size_t i;
+
+	/* Sanity check. */
+	if ((print_hashes == 0) && (bsdtar->ntapes > 0)) {
+		bsdtar_warnc(bsdtar, 0, "--list-archives: can only use -f"
+		    " with --hashes");
+		goto err1;
+	}
 
 	/* Open the archive set for statistics purposes. */
 	if ((d = statstape_open(bsdtar->machinenum, NULL)) == NULL)
 		goto err1;
 
 	/* Ask for the list of archives to be printed. */
-	if (statstape_printlist(d, bsdtar->verbose))
-		goto err2;
+	if (bsdtar->ntapes == 0) {
+		if (statstape_printlist(d, bsdtar->verbose,
+		    bsdtar->option_null_output, print_hashes))
+			goto err2;
+	} else {
+		/* User wants metadata about specific archive(s). */
+		for (i = 0; i < bsdtar->ntapes; i++) {
+			/* Convert ascii hex to a hash. */
+			if (unhexify(bsdtar->tapenames[i], hash, 32)) {
+				bsdtar_warnc(bsdtar, 0,
+				    "Invalid archive hash: %s",
+				    bsdtar->tapenames[i]);
+				goto err1;
+			}
+
+			/* Print desired metadata about the archive. */
+			if (statstape_printlist_item(d, hash,
+			    bsdtar->verbose, bsdtar->option_null_output, 1))
+				goto err2;
+		}
+	}
 
 	/* We're done.  Close the archive set. */
 	if (statstape_close(d))
@@ -205,7 +238,8 @@ void
 tarsnap_mode_fsck(struct bsdtar *bsdtar, int prune, int whichkey)
 {
 
-	if (fscktape(bsdtar->machinenum, bsdtar->cachedir, prune, whichkey)) {
+	if (fscktape(bsdtar->machinenum, bsdtar->cachedir, prune, whichkey,
+	    &bsdtar->storage_modified)) {
 		bsdtar_warnc(bsdtar, 0, "Error fscking archives");
 		goto err0;
 	}
@@ -277,7 +311,7 @@ tarsnap_mode_nuke(struct bsdtar *bsdtar)
 		goto err0;
 	}
 
-	if (nuketape(bsdtar->machinenum)) {
+	if (nuketape(bsdtar->machinenum, &bsdtar->storage_modified)) {
 		bsdtar_warnc(bsdtar, 0, "Error nuking archives");
 		goto err0;
 	}
@@ -298,7 +332,8 @@ void
 tarsnap_mode_recover(struct bsdtar *bsdtar, int whichkey)
 {
 
-	if (recovertape(bsdtar->machinenum, bsdtar->cachedir, whichkey))
+	if (recovertape(bsdtar->machinenum, bsdtar->cachedir, whichkey,
+	    &bsdtar->storage_modified))
 		goto err1;
 
 	/* Success! */
