@@ -1,5 +1,5 @@
-#ifndef _CHUNKIFY_H_
-#define _CHUNKIFY_H_
+#ifndef CHUNKIFY_H_
+#define CHUNKIFY_H_
 
 #include <stdint.h>
 #include <stdlib.h>
@@ -44,6 +44,8 @@ CHUNKIFIER * chunkify_init(uint32_t, uint32_t, chunkify_callback *, void *);
  *
  * The value returned is zero, or the first nonzero value returned by the
  * callback function.
+ *
+ * If ${c} is NULL, do nothing.
  */
 int chunkify_write(CHUNKIFIER *, const uint8_t *, size_t);
 
@@ -54,6 +56,8 @@ int chunkify_write(CHUNKIFIER *, const uint8_t *, size_t);
  *
  * The value returned is zero or the nonzero value returned by the callback
  * function.
+ *
+ * If ${c} is NULL, do nothing.
  */
 int chunkify_end(CHUNKIFIER *);
 
@@ -64,4 +68,4 @@ int chunkify_end(CHUNKIFIER *);
  */
 void chunkify_free(CHUNKIFIER *);
 
-#endif /* !_CHUNKIFY_H_ */
+#endif /* !CHUNKIFY_H_ */
