@@ -1,5 +1,5 @@
-#ifndef _MULTITAPE_H_
-#define _MULTITAPE_H_
+#ifndef MULTITAPE_H_
+#define MULTITAPE_H_
 
 #include <stddef.h>
 #include <stdint.h>
@@ -50,12 +50,14 @@ int readtape_close(TAPE_R *);
 
 /**
  * writetape_open(machinenum, cachedir, tapename, argc, argv, printstats,
- *     dryrun, creationtime, csv_filename):
+ *     dryrun, creationtime, csv_filename, storage_modified):
  * Create a tape with the given name, and return a cookie which can be used
- * for accessing it.  The argument vector must be long-lived.
+ * for accessing it.  The argument vector must be long-lived.  If ${dryrun}
+ * is 2, do not pass any data to the chunkifier or chunk layer; and in this
+ * case, ${printstats} cannot be non-zero.
  */
 TAPE_W * writetape_open(uint64_t, const char *, const char *, int, char **,
-    int, int, time_t, const char *);
+    int, int, time_t, const char *, int *);
 
 /**
  * writetape_setcallbacks(d, callback_chunk, callback_trailer,
@@ -133,15 +135,16 @@ TAPE_D * deletetape_init(uint64_t);
 
 /**
  * deletetape(d, machinenum, cachedir, tapename, printstats, withname,
- *     csv_filename):
+ *     csv_filename, storage_modified):
  * Delete the specified tape, and print statistics to stderr if requested.
  * If ${withname} is non-zero, print statistics with the archive name, not
  * just as "This archive".  Return 0 on success, 1 if the tape does not exist,
  * or -1 on other errors.  If ${csv_filename} is specified, output in CSV
- * format instead of to stderr.
+ * format instead of to stderr.  If the data on the server has been modified,
+ * set ${*storage_modified} to 1.
  */
 int deletetape(TAPE_D *, uint64_t, const char *, const char *, int, int,
-    const char *);
+    const char *, int *);
 
 /**
  * deletetape_free(d):
@@ -157,36 +160,53 @@ void deletetape_free(TAPE_D *);
 TAPE_S * statstape_open(uint64_t, const char *);
 
 /**
- * statstape_printglobal(d, csv_filename):
+ * statstape_printglobal(d, csv_filename, print_nulls):
  * Print global statistics relating to a set of archives.  If ${csv_filename}
- * is not NULL, output will be written in CSV format to that filename.
+ * is not NULL, output will be written in CSV format to that filename.  If
+ * ${print_nulls} is non-zero, use '\0' as separators.
  */
-int statstape_printglobal(TAPE_S *, const char *);
+int statstape_printglobal(TAPE_S *, const char *, int);
 
 /**
- * statstape_printall(d, csv_filename):
+ * statstape_printall(d, csv_filename, print_nulls):
  * Print statistics relating to each of the archives in a set.  If
  * ${csv_filename} is not NULL, output will be written in CSV format to that
- * filename.
+ * filename.  If ${print_nulls} is non-zero, use '\0' as separators.
  */
-int statstape_printall(TAPE_S *, const char *);
+int statstape_printall(TAPE_S *, const char *, int);
 
 /**
- * statstape_printlist(d, verbose):
+ * statstape_printlist_item(d, tapehash, verbose, print_nulls, print_hash):
+ * Print the name of the archive with ${tapehash}.  If ${verbose} > 0, print
+ * the creation times; if ${verbose} > 1, print the argument vector of the
+ * program invocation which created the archive.  If ${print_nulls} > 0, print
+ * null character(s) between archives names and fields instead of newlines,
+ * tabs, and spaces.  If ${print_hash} > 0 and ${verbose} is 0, print the hash
+ * instead of the archive name.  If ${print_hash} > 0 and ${verbose} > 0,
+ * print hash in addition to the normal behaviour.
+ */
+int statstape_printlist_item(TAPE_S *, const uint8_t[32], int, int, int);
+
+/**
+ * statstape_printlist(d, verbose, print_nulls, print_hashes):
  * Print the names of each of the archives in a set.  If ${verbose} > 0, print
  * the creation times; if ${verbose} > 1, print the argument vector of the
- * program invocation which created the archive.
+ * program invocation which created the archive.  If ${print_nulls} > 0, print
+ * null character(s) between archives names and fields instead of newlines,
+ * tabs, and spaces.  If ${print_hashes} > 0 and ${verbose} is 0, print hashes
+ * instead of archive names.  If ${print_hashes} > 0 and ${verbose} > 0, print
+ * hashes in addition to the normal behaviour.
  */
-int statstape_printlist(TAPE_S *, int);
+int statstape_printlist(TAPE_S *, int, int, int);
 
 /**
- * statstape_print(d, tapename, csv_filename):
+ * statstape_print(d, tapename, csv_filename, print_nulls):
  * Print statistics relating to a specific archive in a set.  Return 0 on
  * success, 1 if the tape does not exist, or -1 on other errors.  If
  * ${csv_filename} is not NULL, output will be written in CSV format to that
- * filename.
+ * filename.  If ${print_nulls} is non-zero, use '\0' as separators.
  */
-int statstape_print(TAPE_S *, const char *, const char *);
+int statstape_print(TAPE_S *, const char *, const char *, int);
 
 /**
  * statstape_close(d):
@@ -195,14 +215,15 @@ int statstape_print(TAPE_S *, const char *, const char *);
 int statstape_close(TAPE_S *);
 
 /**
- * fscktape(machinenum, cachedir, prune, whichkey):
+ * fscktape(machinenum, cachedir, prune, whichkey, storage_modified):
  * Correct any inconsistencies in the archive set (by removing orphaned or
  * corrupt files) and reconstruct the chunk directory in ${cachedir}.  If
  * ${prune} is zero, don't correct inconsistencies; instead, exit with an
  * error.  If ${whichkey} is zero, use the write key (for non-pruning fsck
- * only); otherwise, use the delete key.
+ * only); otherwise, use the delete key.  If the data on the server has been
+ * modified, set ${*storage_modified} to 1.
  */
-int fscktape(uint64_t, const char *, int, int);
+int fscktape(uint64_t, const char *, int, int, int *);
 
 /**
  * statstape_initialize(machinenum, cachedir):
@@ -213,17 +234,19 @@ int fscktape(uint64_t, const char *, int, int);
 int statstape_initialize(uint64_t, const char *);
 
 /**
- * recovertape(machinenum, cachedir, whichkey):
+ * recovertape(machinenum, cachedir, whichkey, storage_modified):
  * Complete any pending checkpoint or commit, including a checkpoint in a
  * write transaction being performed by a different machine (if any).  If
  * ${whichkey} is zero, use the write key; otherwise, use the delete key.
+ * If the data on the server has been modified, set ${*storage_modified} to 1.
  */
-int recovertape(uint64_t, const char *, int);
+int recovertape(uint64_t, const char *, int, int *);
 
 /**
- * nuketape(machinenum):
- * Delete all files in the archive set.
+ * nuketape(machinenum, storage_modified):
+ * Delete all files in the archive set.  If the data on the server has been
+ * modified, set ${*storage_modified} to 1.
  */
-int nuketape(uint64_t);
+int nuketape(uint64_t, int *);
 
-#endif /* !_MULTITAPE_H_ */
+#endif /* !MULTITAPE_H_ */
