@@ -10,6 +10,7 @@
 #include "crypto.h"
 #include "multitape_internal.h"
 #include "storage.h"
+#include "warnp.h"
 
 #include "multitape.h"
 
@@ -62,17 +63,18 @@ err0:
 
 /**
  * deletetape(d, machinenum, cachedir, tapename, printstats, withname,
- *     csv_filename):
+ *     csv_filename, storage_modified):
  * Delete the specified tape, and print statistics to stderr if requested.
  * If ${withname} is non-zero, print statistics with the archive name, not
  * just as "This archive".  Return 0 on success, 1 if the tape does not exist,
  * or -1 on other errors.  If ${csv_filename} is specified, output in CSV
- * format instead of to stderr.
+ * format instead of to stderr.  If the data on the server has been modified,
+ * set ${*storage_modified} to 1.
  */
 int
 deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
     const char * tapename, int printstats, int withname,
-    const char * csv_filename)
+    const char * csv_filename, int * storage_modified)
 {
 	struct tapemetadata tmd;
 	CHUNKS_D * C;		/* Chunk layer delete cookie. */
@@ -94,7 +96,7 @@ deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
 		goto err0;
 
 	/* Make sure the lower layers are in a clean state. */
-	if (multitape_cleanstate(cachedir, machinenum, 1))
+	if (multitape_cleanstate(cachedir, machinenum, 1, storage_modified))
 		goto err1;
 
 	/* Get sequence number (# of last committed transaction). */
@@ -108,7 +110,7 @@ deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
 		goto err2;
 
 	/* Cache up to 100 bytes of blocks per chunk in the directory. */
-	storage_read_cache_limit(SR, 100 * chunks_delete_getdirsz(C));
+	storage_read_set_cache_limit(SR, 100 * chunks_delete_getdirsz(C));
 
 	/* Read archive metadata. */
 	switch (multitape_metadata_get_byname(SR, NULL, &tmd, tapename, 0)) {
@@ -150,8 +152,10 @@ deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
 		    withname ? tapename : NULL, csv))
 			goto err4;
 
-		if (csv && fclose(output))
+		if (csv && fclose(output)) {
+			warnp("fclose");
 			goto err3;
+		}
 	}
 
 	/* Close storage and chunk layer cookies. */
@@ -161,11 +165,12 @@ deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
 		goto err1;
 
 	/* Commit the transaction. */
-	if (multitape_commit(cachedir, machinenum, seqnum, 1))
+	if (multitape_commit(cachedir, machinenum, seqnum, 1, storage_modified))
 		goto err1;
 
 	/* Unlock the cache directory. */
-	close(lockfd);
+	if (close(lockfd))
+		warnp("close");
 
 	/* Success! */
 	return (0);
@@ -173,14 +178,15 @@ deletetape(TAPE_D * d, uint64_t machinenum, const char * cachedir,
 err5:
 	multitape_metadata_free(&tmd);
 err4:
-	if (output != stderr)
-		fclose(output);
+	if ((output != stderr) && fclose(output))
+		warnp("fclose");
 err3:
 	chunks_delete_free(C);
 err2:
 	storage_delete_free(S);
 err1:
-	close(lockfd);
+	if (close(lockfd))
+		warnp("close");
 err0:
 	/* Failure! */
 	return (rc);
