@@ -1,4 +1,6 @@
-#include "bsdtar_platform.h"
+#include "platform.h"
+
+#include <sys/types.h>
 
 #include <assert.h>
 #include <limits.h>
