@@ -87,8 +87,11 @@ struct multitape_write_internal {
 
 	/* Callbacks to the chunkification cache. */
 	void * callback_cookie;
-	int (*callback_chunk)(void *, struct chunkheader *);
-	int (*callback_trailer)(void *, const uint8_t *, size_t);
+	int (* callback_chunk)(void *, struct chunkheader *);
+	int (* callback_trailer)(void *, const uint8_t *, size_t);
+
+	/* Used for communication with the storage layer. */
+	int * storage_modified;
 };
 
 static int tapepresent(STORAGE_W *, const char *, const char *);
@@ -103,13 +106,17 @@ static int flushtape(TAPE_W *, int);
 
 /* Initialize stream. */
 static int
-stream_init(struct stream * S, chunkify_callback callback, void * cookie)
+stream_init(struct stream * S, chunkify_callback callback, void * cookie,
+    int no_chunkifier)
 {
 
 	/* Create chunkifier. */
-	if ((S->c =
-	    chunkify_init(MEANCHUNK, MAXCHUNK, callback, cookie)) == NULL)
-		goto err0;
+	if (!no_chunkifier) {
+		if ((S->c = chunkify_init(MEANCHUNK, MAXCHUNK, callback,
+		    cookie)) == NULL)
+			goto err0;
+	} else
+		S->c = NULL;
 
 	/* Allocate elastic array to hold chunk headers. */
 	if ((S->index = chunklist_init(0)) == NULL)
@@ -398,18 +405,28 @@ err0:
 
 /**
  * writetape_open(machinenum, cachedir, tapename, argc, argv, printstats,
- *     dryrun, creationtime, csv_filename):
+ *     dryrun, creationtime, csv_filename, storage_modified):
  * Create a tape with the given name, and return a cookie which can be used
- * for accessing it.  The argument vector must be long-lived.
+ * for accessing it.  The argument vector must be long-lived.  If ${dryrun}
+ * is 2, do not pass any data to the chunkifier or chunk layer; and in this
+ * case, ${printstats} cannot be non-zero.
  */
 TAPE_W *
 writetape_open(uint64_t machinenum, const char * cachedir,
     const char * tapename, int argc, char ** argv, int printstats,
-    int dryrun, time_t creationtime, const char * csv_filename)
+    int dryrun, time_t creationtime, const char * csv_filename,
+    int * storage_modified)
 {
 	struct multitape_write_internal * d;
 	uint8_t lastseq[32];
 	size_t argvlen;
+	int no_chunkifiers;
+
+	/* Sanity check options. */
+	assert(!((dryrun == 2) && printstats));
+
+	/* We don't want the chunkifiers in this case. */
+	no_chunkifiers = (dryrun == 2);
 
 	/* Allocate memory. */
 	if ((d = malloc(sizeof(struct multitape_write_internal))) == NULL)
@@ -462,7 +479,8 @@ writetape_open(uint64_t machinenum, const char * cachedir,
 		goto err3;
 
 	/* If this isn't a dry run, finish any pending commit. */
-	if ((dryrun == 0) && multitape_cleanstate(cachedir, machinenum, 0))
+	if ((dryrun == 0) && multitape_cleanstate(cachedir, machinenum, 0,
+	    storage_modified))
 		goto err4;
 
 	/* If this isn't a dry run, get the sequence number. */
@@ -496,11 +514,11 @@ writetape_open(uint64_t machinenum, const char * cachedir,
 		goto err6;
 
 	/* Initialize streams. */
-	if (stream_init(&d->h, &callback_h, (void *)d))
+	if (stream_init(&d->h, &callback_h, (void *)d, no_chunkifiers))
 		goto err6;
-	if (stream_init(&d->c, &callback_c, (void *)d))
+	if (stream_init(&d->c, &callback_c, (void *)d, no_chunkifiers))
 		goto err7;
-	if (stream_init(&d->t, &callback_t, (void *)d))
+	if (stream_init(&d->t, &callback_t, (void *)d, no_chunkifiers))
 		goto err8;
 
 	/* Initialize header buffer. */
@@ -508,13 +526,19 @@ writetape_open(uint64_t machinenum, const char * cachedir,
 		goto err9;
 
 	/* Initialize file chunkifier. */
-	if ((d->c_file = chunkify_init(MEANCHUNK, MAXCHUNK, &callback_file,
-	    (void *)d)) == NULL)
-		goto err10;
+	if (!no_chunkifiers) {
+		if ((d->c_file = chunkify_init(MEANCHUNK, MAXCHUNK,
+		    &callback_file, (void *)d)) == NULL)
+			goto err10;
+	} else
+		d->c_file = NULL;
 
 	/* No data has entered or exited c_file. */
 	d->c_file_in = d->c_file_out = 0;
 
+	/* Set up communication with the storage layer. */
+	d->storage_modified = storage_modified;
+
 	/* Success! */
 	return (d);
 
@@ -531,8 +555,8 @@ err6:
 err5:
 	storage_write_free(d->S);
 err4:
-	if (d->lockfd != -1)
-		close(d->lockfd);
+	if ((d->lockfd != -1) && close(d->lockfd))
+		warnp("close");
 err3:
 	free(d->cachedir);
 err2:
@@ -951,8 +975,10 @@ writetape_close(TAPE_W * d)
 			goto err2;
 		if (chunks_write_printstats(output, d->C, csv))
 			goto err3;
-		if (csv && fclose(output))
+		if (csv && fclose(output)) {
+			warnp("fclose");
 			goto err2;
+		}
 	}
 
 	/* Ask the chunks layer to prepare for a checkpoint. */
@@ -972,13 +998,14 @@ writetape_close(TAPE_W * d)
 		if (multitape_checkpoint(d->cachedir, d->machinenum,
 		    d->seqnum))
 			goto err1;
-		if (multitape_cleanstate(d->cachedir, d->machinenum, 0))
+		if (multitape_cleanstate(d->cachedir, d->machinenum, 0,
+		    d->storage_modified))
 			goto err1;
 	}
 
 	/* Unlock the cache directory (if applicable). */
-	if (d->lockfd != -1)
-		close(d->lockfd);
+	if ((d->lockfd != -1) && close(d->lockfd))
+		warnp("close");
 
 	/* Free memory. */
 	chunkify_free(d->c_file);
@@ -994,14 +1021,14 @@ writetape_close(TAPE_W * d)
 	return (0);
 
 err3:
-	if (output != stderr)
-		fclose(output);
+	if ((output != stderr) && fclose(output))
+		warnp("fclose");
 err2:
 	chunks_write_free(d->C);
 	storage_write_free(d->S);
 err1:
-	if (d->lockfd != -1)
-		close(d->lockfd);
+	if ((d->lockfd != -1) && close(d->lockfd))
+		warnp("close");
 	chunkify_free(d->c_file);
 	bytebuf_free(d->hbuf);
 	stream_free(&d->t);
@@ -1029,8 +1056,8 @@ writetape_free(TAPE_W * d)
 
 	chunks_write_free(d->C);
 	storage_write_free(d->S);
-	if (d->lockfd != -1)
-		close(d->lockfd);
+	if ((d->lockfd != -1) && close(d->lockfd))
+		warnp("close");
 	chunkify_free(d->c_file);
 	bytebuf_free(d->hbuf);
 	stream_free(&d->t);
