@@ -1,5 +1,5 @@
 /*-
- * Copyright 2006-2008 Colin Percival
+ * Copyright 2006-2025 Tarsnap Backup Inc.
  * All rights reserved.
  *
  * Portions of the file below are covered by the following license:
@@ -53,7 +53,7 @@
 struct tree;
 
 /* Initiate/terminate a tree traversal. */
-struct tree *tree_open(const char * /* pathname */);
+struct tree *tree_open(const char * /* pathname */, int /* noatime */);
 int tree_close(struct tree *);
 
 /*
@@ -71,7 +71,8 @@ int tree_close(struct tree *);
  * fatal error, but it does imply that the relevant subtree won't be
  * visited.  TREE_ERROR_FATAL is returned for an error that left the
  * traversal completely hosed.  Right now, this is only returned for
- * chdir() failures during ascent.
+ * chdir() failures during ascent, or readdir() failures when looking
+ * for the next entry.
  */
 #define	TREE_REGULAR	1
 #define	TREE_POSTDESCENT	2
@@ -133,6 +134,3 @@ int tree_current_is_physical_dir(struct tree *);
 int tree_current_is_physical_link(struct tree *);
 /* "is_dir" is equivalent to S_ISDIR(tree_current_stat()->st_mode) */
 int tree_current_is_dir(struct tree *);
-
-/* For testing/debugging: Dump the internal status to the given filehandle. */
-void tree_dump(struct tree *, FILE *);
