@@ -1,5 +1,5 @@
 /*-
- * Copyright 2006-2008 Colin Percival
+ * Copyright 2006-2025 Tarsnap Backup Inc.
  * All rights reserved.
  *
  * Portions of the file below are covered by the following license:
@@ -63,6 +63,10 @@ __FBSDID("$FreeBSD: src/usr.bin/tar/util.c,v 1.23 2008/12/15 06:00:25 kientzle E
 #define iswprint isprint
 #endif
 
+#include <assert.h>
+
+#include "print_separator.h"
+
 #include "bsdtar.h"
 
 static void	bsdtar_vwarnc(struct bsdtar *, int code,
@@ -296,10 +300,13 @@ process_lines(struct bsdtar *bsdtar, const char *pathname,
 	FILE *f;
 	char *buff, *buff_end, *line_start, *line_end, *p;
 	size_t buff_length, new_buff_length, bytes_read, bytes_wanted;
+	size_t buff_end_pos;
+	size_t line_end_pos;
 	const char * separator;
 	size_t seplen;
 	int lastcharwasr = 0;
 	int ret;
+	int fread_errno;
 
 	if (null) {
 		separator = "";
@@ -333,6 +340,7 @@ process_lines(struct bsdtar *bsdtar, const char *pathname,
 		/* Get some more data into the buffer. */
 		bytes_wanted = buff + buff_length - buff_end;
 		bytes_read = fread(buff_end, 1, bytes_wanted, f);
+		fread_errno = errno;
 		buff_end += bytes_read;
 		/* Process all complete lines in the buffer. */
 		while (line_end < buff_end) {
@@ -358,10 +366,13 @@ process_lines(struct bsdtar *bsdtar, const char *pathname,
 			} else
 				line_end++;
 		}
-		if (feof(f))
+		if (feof(f)) {
+			/* fread() should not set EOF unless this is true. */
+			assert(bytes_read < bytes_wanted);
 			break;
+		}
 		if (ferror(f))
-			bsdtar_errc(bsdtar, 1, errno,
+			bsdtar_errc(bsdtar, 1, fread_errno,
 			    "Can't read %s", pathname);
 		if (line_start > buff) {
 			/* Move a leftover fractional line to the beginning. */
@@ -376,12 +387,14 @@ process_lines(struct bsdtar *bsdtar, const char *pathname,
 				bsdtar_errc(bsdtar, 1, ENOMEM,
 				    "Line too long in %s", pathname);
 			buff_length = new_buff_length;
+			buff_end_pos = buff_end - buff;
+			line_end_pos = line_end - buff;
 			p = realloc(buff, buff_length);
 			if (p == NULL)
 				bsdtar_errc(bsdtar, 1, ENOMEM,
 				    "Line too long in %s", pathname);
-			buff_end = p + (buff_end - buff);
-			line_end = p + (line_end - buff);
+			buff_end = p + buff_end_pos;
+			line_end = p + line_end_pos;
 			line_start = buff = p;
 			bsdtar->conffile_buffer = buff;
 		}
@@ -709,20 +722,25 @@ list_item_verbose(struct bsdtar *bsdtar, FILE *out, struct archive_entry *entry)
 	}
 	if (!now)
 		time(&now);
-	fprintf(out, "%s %d ",
-	    archive_entry_strmode(entry),
-	    (int)(st->st_nlink));
+	fprintf(out, "%s", archive_entry_strmode(entry));
+	print_separator(out, " ", bsdtar->option_null_output, 2);
+	fprintf(out, "%d", (int)(st->st_nlink));
+	print_separator(out, " ", bsdtar->option_null_output, 2);
 
 	/* Use uname if it's present, else uid. */
 	p = archive_entry_uname(entry);
 	if ((p == NULL) || (*p == '\0')) {
-		sprintf(tmp, "%lu ", (unsigned long)st->st_uid);
+		sprintf(tmp, "%lu", (unsigned long)st->st_uid);
 		p = tmp;
 	}
 	w = strlen(p);
 	if (w > bsdtar->u_width)
 		bsdtar->u_width = w;
-	fprintf(out, "%-*s ", (int)bsdtar->u_width, p);
+	if (bsdtar->option_null_output)
+		fprintf(out, "%s", p);
+	else
+		fprintf(out, "%-*s", (int)bsdtar->u_width, p);
+	print_separator(out, " ", bsdtar->option_null_output, 2);
 
 	/* Use gname if it's present, else gid. */
 	p = archive_entry_gname(entry);
@@ -755,7 +773,10 @@ list_item_verbose(struct bsdtar *bsdtar, FILE *out, struct archive_entry *entry)
 	}
 	if (w + strlen(tmp) >= bsdtar->gs_width)
 		bsdtar->gs_width = w+strlen(tmp)+1;
-	fprintf(out, "%*s", (int)(bsdtar->gs_width - w), tmp);
+	if (bsdtar->option_null_output)
+		fprintf(out, "%s", tmp);
+	else
+		fprintf(out, "%*s", (int)(bsdtar->gs_width - w), tmp);
 
 	/* Format the time. */
 	tim = (time_t)st->st_mtime;
@@ -777,13 +798,21 @@ list_item_verbose(struct bsdtar *bsdtar, FILE *out, struct archive_entry *entry)
 #endif
 	}
 	strftime(tmp, sizeof(tmp), fmt, localtime(&tim));
-	fprintf(out, " %s ", tmp);
+	print_separator(out, " ", bsdtar->option_null_output, 2);
+	fprintf(out, "%s", tmp);
+	print_separator(out, " ", bsdtar->option_null_output, 2);
 	safe_fprintf(out, "%s", archive_entry_pathname(entry));
 
 	/* Extra information for links. */
-	if (archive_entry_hardlink(entry)) /* Hard link */
-		safe_fprintf(out, " link to %s",
-		    archive_entry_hardlink(entry));
-	else if (S_ISLNK(st->st_mode)) /* Symbolic link */
-		safe_fprintf(out, " -> %s", archive_entry_symlink(entry));
+	if (archive_entry_hardlink(entry)) { /* Hard link */
+		print_separator(out, " ", bsdtar->option_null_output, 2);
+		fprintf(out, "link to");
+		print_separator(out, " ", bsdtar->option_null_output, 2);
+		safe_fprintf(out, "%s", archive_entry_hardlink(entry));
+	} else if (S_ISLNK(st->st_mode)) { /* Symbolic link */
+		print_separator(out, " ", bsdtar->option_null_output, 2);
+		fprintf(out, "->");
+		print_separator(out, " ", bsdtar->option_null_output, 2);
+		safe_fprintf(out, "%s", archive_entry_symlink(entry));
+	}
 }
